import { __decorate } from 'tslib';
import { HostListener, Directive, EventEmitter, ElementRef, Input, Output, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

let myDoc = document;
myDoc.elementsFromPoint = myDoc.elementsFromPoint || function (x, y) {
    let element;
    const old_visibility = [];
    const elements = [];
    while (true) {
        element = document.elementFromPoint(x, y);
        if (!element || element === document.documentElement) {
            break;
        }
        elements.push(element);
        old_visibility.push(element.style.visibility);
        element.style.visibility = 'hidden'; // Temporarily hide the element (without changing the layout)
    }
    for (let k = 0; k < elements.length; k++) {
        elements[k].style.visibility = old_visibility[k];
    }
    return elements;
};

/*
interface ShadowRoot extends DocumentFragment {
    styleSheets     : StyleSheetList;
    innerHTML       : string;
    host            : Element;
    activeElement   : Element;
    elementFromPoint        (x : number, y : number) : Element;
    elementsFromPoint       (x : number, y : number) : Element[];
    caretPositionFromPoint  (x : number, y : number); // => CaretPosition
};

interface ElementWithShadowRoot extends HTMLElement {
    shadowRoot  : ShadowRoot;
};*/
const dragPointerId = 'dragPointer';
class DragManager {
    constructor() {
        this.draggingPointer = new Map();
        this.draggedStructures = new Map();
        this.dropZones = new Map();
    }
    preStartDrag(idPointer, x, y, delay, dist) {
        this.draggingPointer.set(idPointer, { x: x, y: y });
        return new Promise((resolve, reject) => {
            setTimeout(() => {
                const ptr = this.draggingPointer.get(idPointer);
                const gogo = ptr && (Math.abs(x - ptr.x) + Math.abs(y - ptr.y)) < dist;
                this.draggingPointer.delete(idPointer);
                gogo ? resolve() : reject();
            }, Math.max(0, delay));
        });
    }
    startDrag(idPointer, dragged) {
        this.draggedStructures.set(idPointer, dragged);
        const possibleDropZones = new Map();
        this.dropZones.forEach(dz => {
            if (dz.checkAccept(dragged)) {
                dz.appendDropCandidatePointer(idPointer);
                possibleDropZones.set(dz.root, dz);
            }
        });
        return possibleDropZones;
    }
    isAssociatedToDropZone(element) {
        return this.dropZones.has(element);
    }
    registerDropZone(dropzone) {
        this.dropZones.set(dropzone.root, dropzone);
    }
    unregisterDropZone(dropzone) {
        this.dropZones.delete(dropzone.root);
    }
    pointerMove(idPointer, x, y) {
        const ptr = this.draggingPointer.get(idPointer);
        if (ptr) {
            ptr.x = x;
            ptr.y = y;
        }
        const dragged = this.draggedStructures.get(idPointer);
        if (dragged && dragged instanceof Draggable) {
            dragged.move(x, y);
        }
        return dragged !== undefined;
    }
    pointerRelease(idPointer) {
        const dragged = this.draggedStructures.get(idPointer);
        if (dragged) {
            if (dragged instanceof Draggable) {
                dragged.stop();
            }
        }
        this.draggedStructures.delete(idPointer);
        this.draggingPointer.delete(idPointer);
        return dragged !== undefined;
    }
}
const DM = new DragManager();
let dragDropInit = false;
let DragDrop = class DragDrop {
    constructor() {
        this.nbDragEnter = 0;
        if (dragDropInit) {
            console.error('Do not create multiple instance of directive drag-and-drop-wrapper !');
        }
        else {
            dragDropInit = true;
        }
    }
    removeFeedbackForDragPointer() {
        this.nbDragEnter = 0;
        DM.dropZones.forEach(dz => {
            dz.removePointerHover(dragPointerId);
            dz.removeDropCandidatePointer(dragPointerId);
        });
    }
    drop(e) {
        e.preventDefault();
        e.stopPropagation();
        this.removeFeedbackForDragPointer();
    }
    dragover(e) {
        e.preventDefault();
        e.stopPropagation();
    }
    dragenter(e) {
        this.nbDragEnter++;
        if (this.nbDragEnter === 1) {
            DM.startDrag(dragPointerId, e);
        }
    }
    dragleave(e) {
        this.nbDragEnter--;
        if (this.nbDragEnter === 0) {
            this.removeFeedbackForDragPointer();
            DM.pointerRelease(dragPointerId);
        }
    }
    dragend(e) {
        DM.pointerRelease(dragPointerId);
        this.removeFeedbackForDragPointer();
        e.preventDefault();
    }
    mousemove(e) {
        DM.pointerMove('mouse', e.clientX, e.clientY);
    }
    mouseup(e) {
        DM.pointerRelease('mouse');
    }
    touchmove(e) {
        for (let i = 0; i < e.changedTouches.length; i++) {
            const touch = e.changedTouches.item(i);
            if (DM.pointerMove(touch.identifier.toString(), touch.clientX, touch.clientY)) {
                e.preventDefault();
                e.stopPropagation();
            }
        }
    }
    touchend(e) {
        for (let i = 0; i < e.changedTouches.length; i++) {
            const touch = e.changedTouches.item(i);
            if (DM.pointerRelease(touch.identifier.toString())) {
                e.preventDefault();
                e.stopPropagation();
            }
        }
    }
};
__decorate([
    HostListener('document: drop', ['$event'])
], DragDrop.prototype, "drop", null);
__decorate([
    HostListener('document: dragover', ['$event'])
], DragDrop.prototype, "dragover", null);
__decorate([
    HostListener('document: dragenter', ['$event'])
], DragDrop.prototype, "dragenter", null);
__decorate([
    HostListener('document: dragleave', ['$event'])
], DragDrop.prototype, "dragleave", null);
__decorate([
    HostListener('document: dragend', ['$event'])
], DragDrop.prototype, "dragend", null);
__decorate([
    HostListener('document: mousemove', ['$event'])
], DragDrop.prototype, "mousemove", null);
__decorate([
    HostListener('document: mouseup', ['$event'])
], DragDrop.prototype, "mouseup", null);
__decorate([
    HostListener('document: touchmove', ['$event'])
], DragDrop.prototype, "touchmove", null);
__decorate([
    HostListener('document: touchend', ['$event'])
], DragDrop.prototype, "touchend", null);
DragDrop = __decorate([
    Directive({
        selector: '*[drag-and-drop-wrapper]'
    })
], DragDrop);
const offsetElement = (element) => {
    let left = 0, top = 0;
    while (element) {
        top += element.offsetTop - element.scrollTop + element.clientTop;
        left += element.offsetLeft - element.scrollLeft + element.clientLeft;
        element = element.offsetParent;
    }
    return { left: left, top: top };
};
const ɵ0 = offsetElement;
let Draggable = class Draggable {
    constructor(el) {
        this.disabled = false;
        this.onDragStart = new EventEmitter();
        this.onDragEnd = new EventEmitter();
        this.isBeingDragged = false;
        this.cloneNode = null;
        this.possibleDropZones = new Map();
        this.root = el.nativeElement;
        if (!dragDropInit) {
            console.error('You should add one dragdrop attribute to your code before using draggable');
        }
    }
    ngOnInit() {
        //
    }
    ngOnDestroy() {
        // XXX No stop in case of unplug while dragging... : // this.stop();
    }
    onMouseDown(event) {
        if (!this.disabled) {
            event.preventDefault();
            event.stopPropagation();
            this.prestart('mouse', event.clientX, event.clientY);
        }
    }
    onTouchStart(event) {
        event.stopPropagation();
        for (let i = 0; i < event.changedTouches.length; i++) {
            const touch = event.changedTouches.item(i);
            this.prestart(touch.identifier.toString(), touch.clientX, touch.clientY);
        }
    }
    prestart(idPointer, x, y) {
        DM.preStartDrag(idPointer, x, y, this.startDelay || 0, this.startDistance || 10).then(() => {
            this.start(idPointer, x, y);
        }, () => {
            // console.error('skip the drag');
        });
    }
    start(idPointer, x, y) {
        if (!this.isBeingDragged) {
            this.isBeingDragged = true;
            this.idPointer = idPointer;
            const offset = offsetElement(this.root);
            this.ox = x;
            this.oy = y;
            this.dx = x - offset.left;
            this.dy = y - offset.top;
            this.tx = this.root.offsetWidth;
            this.ty = this.root.offsetHeight;
            this.getClone();
            this.cloneNode.style.left = (x - this.dx) + 'px';
            this.cloneNode.style.top = (y - this.dy) + 'px';
            this.onDragStart.emit(this.draggedData);
            this.possibleDropZones = DM.startDrag(idPointer, this);
        }
    }
    stop() {
        this.possibleDropZones.forEach(dz => {
            dz.removePointerHover(this.idPointer);
            dz.removeDropCandidatePointer(this.idPointer);
        });
        this.isBeingDragged = false;
        this.possibleDropZones.clear();
        this.idPointer = null;
        if (this.currentDropZone) {
            this.currentDropZone.drop(this.draggedData);
        }
        this.currentDropZone = null;
        this.onDragEnd.emit(this.draggedData);
        this.deleteClone();
    }
    move(x, y) {
        let element = null;
        if (this.cloneNode === null) {
            this.getClone();
        }
        if (this.cloneNode) {
            this.cloneNode.style.left = (x - this.dx) + 'px';
            this.cloneNode.style.top = (y - this.dy) + 'px';
            const visibility = this.cloneNode.style.visibility;
            this.cloneNode.style.visibility = 'hidden';
            const top = this.cloneNode.style.top;
            this.cloneNode.style.top = '999999999px';
            element = myDoc.elementFromPoint(x, y);
            this.cloneNode.style.top = top;
            this.cloneNode.style.visibility = visibility;
            const prevDropZone = this.currentDropZone;
            while (element) {
                this.currentDropZone = this.possibleDropZones.get(element);
                if (this.currentDropZone) {
                    break;
                }
                element = element.parentElement;
            }
            if (prevDropZone !== this.currentDropZone) {
                if (prevDropZone) {
                    prevDropZone.removePointerHover(this.idPointer);
                }
                if (this.currentDropZone) {
                    this.currentDropZone.appendPointerHover(this.idPointer);
                }
            }
        }
        return this;
    }
    deepStyle(original, clone) {
        if (original instanceof HTMLElement) {
            const style = window.getComputedStyle(original);
            for (let i = 0; i < style.length; i++) {
                const att = style[i];
                clone.style[att] = style[att];
            }
            for (let i = 0; i < original.children.length; i++) {
                this.deepStyle(original.children.item(i), clone.children.item(i));
            }
        }
    }
    deleteClone() {
        if (this.cloneNode) {
            if (this.cloneNode.parentNode) {
                this.cloneNode.parentNode.removeChild(this.cloneNode);
            }
            this.cloneNode = null;
        }
    }
    getClone() {
        if (this.cloneNode === null) {
            this.cloneNode = this.root.cloneNode(true);
            this.deepStyle(this.root, this.cloneNode);
            document.body.appendChild(this.cloneNode);
            this.cloneNode.style.position = 'absolute';
            this.cloneNode.style.zIndex = '999';
            this.cloneNode.style.marginLeft = '0';
            this.cloneNode.style.marginTop = '0';
            this.cloneNode.style.marginRight = '0';
            this.cloneNode.style.marginBottom = '0';
            this.cloneNode.style.opacity = '';
            this.cloneNode.style.cursor = '';
            this.cloneNode.style.transform = '';
            this.cloneNode.style.transformOrigin = '';
            this.cloneNode.style.animation = '';
            this.cloneNode.style.transition = '';
            this.cloneNode.classList.add('drag-and-drop-clone-node');
        }
        return this.cloneNode;
    }
};
Draggable.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Input('draggable')
], Draggable.prototype, "draggedData", void 0);
__decorate([
    Input('drag-start-delay')
], Draggable.prototype, "startDelay", void 0);
__decorate([
    Input('start-distance')
], Draggable.prototype, "startDistance", void 0);
__decorate([
    Input('disabled')
], Draggable.prototype, "disabled", void 0);
__decorate([
    Output('drag-start')
], Draggable.prototype, "onDragStart", void 0);
__decorate([
    Output('drag-end')
], Draggable.prototype, "onDragEnd", void 0);
__decorate([
    HostListener('mousedown', ['$event'])
], Draggable.prototype, "onMouseDown", null);
__decorate([
    HostListener('touchstart', ['$event'])
], Draggable.prototype, "onTouchStart", null);
Draggable = __decorate([
    Directive({
        selector: '*[draggable]'
    })
], Draggable);
let Dropzone = class Dropzone {
    constructor(el) {
        this.nbDragEnter = 0;
        this.onDropEmitter = new EventEmitter();
        this.onDragStart = new EventEmitter();
        this.onDragEnd = new EventEmitter();
        this.onDragEnter = new EventEmitter();
        this.onDragLeave = new EventEmitter();
        this.dropCandidateofPointers = [];
        this.pointersHover = [];
        if (!dragDropInit) {
            console.error('You should add one dragdrop attribute to your code before using dropzone');
        }
        this.root = el.nativeElement;
        DM.registerDropZone(this);
    }
    ngOnInit() {
        //
    }
    ngOnDestroy() {
        DM.unregisterDropZone(this);
    }
    BrowserDragEnter(event) {
        this.nbDragEnter++;
        if (this.nbDragEnter === 1) {
            this.appendPointerHover(dragPointerId);
        }
    }
    BrowserDragLeave(event) {
        this.nbDragEnter--;
        if (this.nbDragEnter === 0) {
            this.removePointerHover(dragPointerId);
        }
    }
    BrowserDrop(event) {
        DM.pointerRelease(dragPointerId);
        this.nbDragEnter = 0;
        this.onDropEmitter.emit(event);
    }
    drop(obj) {
        this.onDropEmitter.emit(obj);
    }
    checkAccept(drag) {
        let res;
        if (drag instanceof Draggable) {
            res = this.acceptFunction ? this.acceptFunction(drag.draggedData) : true;
        }
        else {
            res = this.acceptFunction ? this.acceptFunction(drag) : true;
        }
        return res;
    }
    hasPointerHover(idPointer) {
        return this.pointersHover.indexOf(idPointer) >= 0;
    }
    appendPointerHover(idPointer) {
        if (this.pointersHover.indexOf(idPointer) === -1) {
            const dragged = DM.draggedStructures.get(idPointer);
            this.pointersHover.push(idPointer);
            if (dragged instanceof Draggable) {
                if (this.dragOverCSS_pointer) {
                    dragged.getClone().classList.add(this.dragOverCSS_pointer);
                }
                this.onDragEnter.emit(dragged.draggedData);
            }
            else {
                this.onDragEnter.emit(dragged);
            }
            if (this.dragOverCSS) {
                this.root.classList.add(this.dragOverCSS);
            }
        }
    }
    removePointerHover(idPointer) {
        const pos = this.pointersHover.indexOf(idPointer);
        if (pos >= 0) {
            const dragged = DM.draggedStructures.get(idPointer);
            this.pointersHover.splice(pos, 1);
            if (dragged instanceof Draggable) {
                if (this.dragOverCSS_pointer) {
                    dragged.getClone().classList.remove(this.dragOverCSS_pointer);
                }
                this.onDragLeave.emit(dragged.draggedData);
            }
            else {
                this.onDragLeave.emit(dragged);
            }
            if (this.pointersHover.length === 0 && this.dragOverCSS) {
                this.root.classList.remove(this.dragOverCSS);
            }
        }
    }
    appendDropCandidatePointer(idPointer) {
        if (this.dropCandidateofPointers.indexOf(idPointer) === -1) {
            const dragged = DM.draggedStructures.get(idPointer);
            if (dragged instanceof Draggable) {
                this.onDragStart.emit(dragged.draggedData);
            }
            else {
                this.onDragStart.emit(dragged);
            }
            this.dropCandidateofPointers.push(idPointer);
            if (this.dragCSS) {
                this.root.classList.add(this.dragCSS);
            }
        }
    }
    removeDropCandidatePointer(idPointer) {
        const pos = this.dropCandidateofPointers.indexOf(idPointer);
        if (pos >= 0) {
            const dragged = DM.draggedStructures.get(idPointer);
            if (dragged instanceof Draggable) {
                this.onDragEnd.emit(dragged.draggedData);
            }
            else {
                this.onDragEnd.emit(dragged);
            }
            this.dropCandidateofPointers.splice(pos, 1);
            if (this.dropCandidateofPointers.length === 0 && this.dragCSS) {
                this.root.classList.remove(this.dragCSS);
            }
        }
    }
};
Dropzone.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Input('drag-css')
], Dropzone.prototype, "dragCSS", void 0);
__decorate([
    Input('drag-over-css')
], Dropzone.prototype, "dragOverCSS", void 0);
__decorate([
    Input('drag-over-css-for-draggable')
], Dropzone.prototype, "dragOverCSS_pointer", void 0);
__decorate([
    Input('accept-function')
], Dropzone.prototype, "acceptFunction", void 0);
__decorate([
    Output('ondrop')
], Dropzone.prototype, "onDropEmitter", void 0);
__decorate([
    Output('drag-start')
], Dropzone.prototype, "onDragStart", void 0);
__decorate([
    Output('drag-end')
], Dropzone.prototype, "onDragEnd", void 0);
__decorate([
    Output('drag-enter')
], Dropzone.prototype, "onDragEnter", void 0);
__decorate([
    Output('drag-leave')
], Dropzone.prototype, "onDragLeave", void 0);
__decorate([
    HostListener('dragenter', ['$event'])
], Dropzone.prototype, "BrowserDragEnter", null);
__decorate([
    HostListener('dragleave', ['$event'])
], Dropzone.prototype, "BrowserDragLeave", null);
__decorate([
    HostListener('drop', ['$event'])
], Dropzone.prototype, "BrowserDrop", null);
Dropzone = __decorate([
    Directive({ selector: '*[dropzone]' })
], Dropzone);

var NgxDragAndDropModule_1;
let NgxDragAndDropModule = NgxDragAndDropModule_1 = class NgxDragAndDropModule {
    static forRoot() {
        return {
            ngModule: NgxDragAndDropModule_1,
            providers: []
        };
    }
};
NgxDragAndDropModule = NgxDragAndDropModule_1 = __decorate([
    NgModule({
        imports: [
            CommonModule,
        ],
        declarations: [
            Draggable,
            Dropzone,
            DragDrop,
        ],
        exports: [
            Draggable,
            Dropzone,
            DragDrop,
        ]
    })
], NgxDragAndDropModule);

/**
 * Generated bundle index. Do not edit.
 */

export { NgxDragAndDropModule, DragDrop as ɵa, Draggable as ɵb, Dropzone as ɵc };
//# sourceMappingURL=ngx-drag-and-drop.js.map
